function Create(self)
	self.charge = 0;
	
	self.soundAddLight = CreateSoundContainer("Add Light Toaster", "FoodFight.rte");
	self.soundAddMedium = CreateSoundContainer("Add Medium Toaster", "FoodFight.rte");
	self.soundAddMax = CreateSoundContainer("Add Max Toaster", "FoodFight.rte");
	
	self.soundChargeStart = CreateSoundContainer("Tick Toaster", "FoodFight.rte");
	self.soundCharge = CreateSoundContainer("Charge Toaster", "FoodFight.rte");
	
	self.minFireVel = 20;
	self.maxFireVel = 40;
	self.spread = math.rad(self.ShakeRange);

	self.shotVariants = 4;
	self.chargeDelay = 2000;

	self.animTimer = Timer();
	self.chargeTimer = Timer();
	self.chargeTimer:SetSimTimeLimitMS(self.chargeDelay);
	--The following timer prevents a glitch where you can fire twice by putting the gun inside the inventory while charging
	self.inventorySwapTimer = Timer();
	self.inventorySwapTimer:SetSimTimeLimitMS(math.ceil(TimerMan.DeltaTimeMS));
	
	--Reload delay
	self.reloadTimer = Timer();
	self.reloadTimer:SetSimTimeLimitMS(500);
	
	self.smokeTimer = Timer();
	self.dispenseToast = function(fired)
		self.soundCharge:Stop(-1);
		
		local toastLevel = self.charge < 0.3 and 1 or (self.charge < 0.5 and 2 or (self.charge < 0.9 and 3 or 4));
		if toastLevel >= 4 then
			self.soundAddMax:Play(self.Pos);
		elseif toastLevel >= 3 then
			self.soundAddMedium:Play(self.Pos);
		elseif toastLevel >= 2 then
			self.soundAddLight:Play(self.Pos);
		end
		for i = 1, 2 do
			local par = CreateAEmitter("FoodFight.rte/Toast Shot " .. toastLevel);
			par.Team = self.Team;
			par.IgnoresTeamHits = true;
			par.Pos = self.MuzzlePos + Vector((1 - i * 2) * self.FlipFactor, 0):RadRotate(self.RotAngle);
			local spread = RangeRand(-self.spread, self.spread);
			local vel = fired and (self.minFireVel + (self.maxFireVel - self.minFireVel) * self.charge) or self.minFireVel * 0.1;
			par.Vel = self.Vel + Vector(vel/(1 + math.abs(spread)) * self.FlipFactor, 0):RadRotate(self.RotAngle + spread);
			par.AngularVel = spread * 25/(1 + self.charge);
			par.RotAngle = par.Vel.AbsRadAngle;
			MovableMan:AddParticle(par);
		end
		self.charge = 0;
		self.activated = false;
		self.SupportOffset = Vector(-5, -6);
	end
end
function Update(self)

	self.soundCharge.Pos = self.Pos;
	if self.activated then
		self:Activate();
		self.inventorySwapTimer:Reset();
	end

	if self.Magazine and self.RoundInMagCount ~= 0 then
		self.Frame = 1;
		self.reloadTimer:Reset();
		if self.inventorySwapTimer:IsPastSimTimeLimit() then
			self.charge = 0;
			self.Magazine.Scale = 1;
		end
		self.inventorySwapTimer:Reset();
		if self.Magazine.RoundCount > 0 then
			if self:DoneReloading() then
				self:Deactivate();
				self.SupportOffset = Vector(3, -5);
			end
			if self:IsActivated() then
				if self.charge == 0 then
					self.soundChargeStart:Play(self.Pos);
					self.soundCharge:Play(self.Pos);
				end
				self.SupportOffset = Vector(-1, -5);
				self.Magazine.Scale = 0;
				self:Deactivate();

				if not self.chargeTimer:IsPastSimTimeLimit() then
					self.charge = self.chargeTimer.ElapsedSimTimeMS/self.chargeDelay;
				else
					self.charge = 1;
					--CPU actor will release the beam at full power
					local parent = self:GetRootParent();
					if parent and IsActor(parent) and not ToActor(parent):IsPlayerControlled() then
						ToActor(parent):GetController():SetState(Controller.WEAPON_FIRE, false);
					elseif self.activated then
						self.activated = false;
					end
					if self.smokeTimer:IsPastSimMS(500) then
						local smoke = CreateMOSParticle((math.random() < 0.5 and "Tiny" or "Small") .. " Smoke Ball 1", "Base.rte");
						smoke.Pos = self.MuzzlePos;
						smoke.Vel = self.Vel * 0.5;
						MovableMan:AddParticle(smoke);
						self.smokeTimer:Reset();
					end
				end
				self.Magazine.RoundCount = math.ceil(self.charge * 10);
			else
				self.Magazine.RoundCount = 1;
				if self.charge > 0 then
					--Trigger gun like normal and dispense the shot
					self:Activate();
				end
				self.chargeTimer:Reset();
			end
		end
	else
		self.Frame = 0;
		local parent = self:GetRootParent();
		if parent and IsActor(parent) and ToActor(parent):IsPlayerControlled() then
			if self.reloadTimer:IsPastSimTimeLimit() then
				ToActor(parent):GetController():SetState(Controller.WEAPON_RELOAD, true);
			else
				ToActor(parent):GetController():SetState(Controller.WEAPON_RELOAD, false);
			end
		end
	end
	if self.FiredFrame then
		self.dispenseToast(true);
		self:Deactivate();
	end
	if self:IsReloading() and self.charge > 0 then
		self.dispenseToast(false);
	end
end

function OnDetach(self, exParent)
	if self.charge > 0 then
		self.activated = true;
	else
		self.soundCharge:Stop(-1);
	end
end

function Destroy(self)
	self.soundCharge:Stop(-1);
end